/*************************************************************
 *           Class Definition "Initiailization"
 *************************************************************
 *  This class initializes the program getting command inputs, 
 * reading initial configurations, topology of the system, 
 * setting MPI, interaction types and domain decomposition.
 * **********************************************************/

#ifndef __INITIALIZATION_ANAL__HPP
#define __INITIALIZATION_ANAL__HPP


#include "../command.hpp"
#include "../topology.hpp"
#include "../control.hpp"
#include "../configuration.hpp"
#include "../interactions.hpp"
#include "../timer.hpp"


namespace dpdanal{


class Initialization{
private:
    dpd::SetMPI *mpi;
    dpd::Command *command;
    dpd::Topology *topol;
    dpd::Control *control;
    dpd::Configuration *config;
    dpd::Decomposition* decomp;
    dpd::Interactions* interactions;
    dpd::Timer* timer;

    bool need_coord;
    bool need_veloc;
    bool need_stres;
    bool need_force;
    bool multicore;
    
    int nbeads;
    int nsteps;
    int begstep;
    int endstep;
    int skipstep;
    int nsteps_proc;

    void printProgramStartingMessage();

public:
    Initialization(){}
    Initialization(dpd::Command* command);
    ~Initialization();

    void checkNumberOfCoresForProgram();
    void getGeneralParameters();
    /*Functions to get a pointer of each instance*/
    dpd::SetMPI* getMPI(){ return mpi; }
    dpd::Command* getCommand(){ return command; }
    dpd::Topology* getTopology(){ return topol;}
    dpd::Control* getControl(){ return control; }
    dpd::Configuration* getConfiguration(){ return config; }
    dpd::Decomposition* getDecomposition(){ return decomp; }
    dpd::Timer* getTimer(){ return timer; }
    dpd::Interactions* getInteractions(){ return interactions; }

    int getNbeads(){ return nbeads; }
    int getNsteps(){ return nsteps; }
    int getBeginStep(){ return begstep; }
    int getEndStep(){ return endstep; }
    int getSkipStep(){ return skipstep; }
    int getNstepsProc(){ return nsteps_proc; }
    bool needMultiCore(){ return multicore; }


};
};

#endif
