#include "polymerlocalconc.hpp"
#include <algorithm>
#include <math.h>
#include <sstream>
#include <iomanip>

using namespace dpd;
PolymerLocalConc::PolymerLocalConc(InitialSet initset):Property(initset){
    title="   Calculation of local concentration of polymer beads";

    dz=1.0;
    numz=int(box[2]/dz)+1;
    nprops=numz;
    for_timeevol=true;
    need_position=true;

    outfile="polymer_localconc.out";


    outtitle_tevol="#Local concentration of polymer beads";
    outheader_tevol=Svec{"#Time"};
    for(int i=0;i<numz;i++){
        outheader_tevol.push_back(std::to_string((i+0.5)*dz));
    }

}





void PolymerLocalConc::initializeVariables(){
    initializeResultArrays();
    return;
}

void PolymerLocalConc::calculateStep(int step){
    ntotbeads=Ivec(numz, 0);
    npolbeads=Ivec(numz, 0);
    for(int i=0;i<nbeads;i++){
        int index=static_cast<int>(particles[i]->coord[2]/dz);
        if(particles[i]->getTypeName().compare("P")==0){
            npolbeads[index]++;
            ntotbeads[index]++;
        }
        else
            ntotbeads[index]++;
    }
    for(int i=0;i<numz;i++){
        tevol[i][step]=static_cast<real>(npolbeads[i])/ntotbeads[i];
    }




    return;
}







