#ifndef __INTERACTIONS__HPP
#define __INTERACTIONS__HPP

#include "control.hpp"
#include "topology.hpp"
#include "mdpdforce.hpp"
#include "dpdforce.hpp"
#include "bondlenharmonic.hpp"
#include "springpulling.hpp"
#include "gravity.hpp"
#include "solidwall.hpp"
#include "removalcomvel.hpp"
#include "boxelongation.hpp"
#include "pinning.hpp"
#include "uniflowpbc.hpp"
#include "slipspringharmonic.hpp"
#include "wallinducedshear.hpp"
namespace dpd{


class Interactions{
private:
    Control* control;
    Topology* topology;
    Configuration* config;
    Decomposition* decomp;
    SetMPI* mpi;
    NonbondedList nblist;
    BondedList bdlist;
    ExternalList exlist;
    ExtensionList extensionlist;
    
    
public:
    Interactions(){}
    Interactions(Control* control, Topology* topology, Configuration* config, Decomposition* decomp);
    ~Interactions(){}

    void addNonbondedInteraction();
    void addBondedInteraction();
    void addExternalInteraction();
    void addExtension();
    void applyExtensionForPosition(int step);
    void applyExtensionForVelocity(int step);
    NonbondedList& getNonbondedInteractions(){ return nblist; }
    BondedList& getBondedInteractions(){ return bdlist; }
    ExternalList& getExternalInteractions(){ return exlist; }
    ExtensionList& getExtension(){ return extensionlist;}
};
};


#endif
